/****************************************************************************
** Meta object code from reading C++ file 'adapter1_bluez5_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/bluez/adapter1_bluez5_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'adapter1_bluez5_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OrgBluezAdapter1Interface_t {
    QByteArrayData data[22];
    char stringdata0[260];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OrgBluezAdapter1Interface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OrgBluezAdapter1Interface_t qt_meta_stringdata_OrgBluezAdapter1Interface = {
    {
QT_MOC_LITERAL(0, 0, 25), // "OrgBluezAdapter1Interface"
QT_MOC_LITERAL(1, 26, 12), // "RemoveDevice"
QT_MOC_LITERAL(2, 39, 19), // "QDBusPendingReply<>"
QT_MOC_LITERAL(3, 59, 0), // ""
QT_MOC_LITERAL(4, 60, 15), // "QDBusObjectPath"
QT_MOC_LITERAL(5, 76, 6), // "device"
QT_MOC_LITERAL(6, 83, 18), // "SetDiscoveryFilter"
QT_MOC_LITERAL(7, 102, 10), // "properties"
QT_MOC_LITERAL(8, 113, 14), // "StartDiscovery"
QT_MOC_LITERAL(9, 128, 13), // "StopDiscovery"
QT_MOC_LITERAL(10, 142, 7), // "Address"
QT_MOC_LITERAL(11, 150, 5), // "Alias"
QT_MOC_LITERAL(12, 156, 5), // "Class"
QT_MOC_LITERAL(13, 162, 12), // "Discoverable"
QT_MOC_LITERAL(14, 175, 19), // "DiscoverableTimeout"
QT_MOC_LITERAL(15, 195, 11), // "Discovering"
QT_MOC_LITERAL(16, 207, 8), // "Modalias"
QT_MOC_LITERAL(17, 216, 4), // "Name"
QT_MOC_LITERAL(18, 221, 8), // "Pairable"
QT_MOC_LITERAL(19, 230, 15), // "PairableTimeout"
QT_MOC_LITERAL(20, 246, 7), // "Powered"
QT_MOC_LITERAL(21, 254, 5) // "UUIDs"

    },
    "OrgBluezAdapter1Interface\0RemoveDevice\0"
    "QDBusPendingReply<>\0\0QDBusObjectPath\0"
    "device\0SetDiscoveryFilter\0properties\0"
    "StartDiscovery\0StopDiscovery\0Address\0"
    "Alias\0Class\0Discoverable\0DiscoverableTimeout\0"
    "Discovering\0Modalias\0Name\0Pairable\0"
    "PairableTimeout\0Powered\0UUIDs"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OrgBluezAdapter1Interface[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
      12,   42, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   34,    3, 0x0a /* Public */,
       6,    1,   37,    3, 0x0a /* Public */,
       8,    0,   40,    3, 0x0a /* Public */,
       9,    0,   41,    3, 0x0a /* Public */,

 // slots: parameters
    0x80000000 | 2, 0x80000000 | 4,    5,
    0x80000000 | 2, QMetaType::QVariantMap,    7,
    0x80000000 | 2,
    0x80000000 | 2,

 // properties: name, type, flags
      10, QMetaType::QString, 0x00095001,
      11, QMetaType::QString, 0x00095103,
      12, QMetaType::UInt, 0x00095001,
      13, QMetaType::Bool, 0x00095103,
      14, QMetaType::UInt, 0x00095103,
      15, QMetaType::Bool, 0x00095001,
      16, QMetaType::QString, 0x00095001,
      17, QMetaType::QString, 0x00095001,
      18, QMetaType::Bool, 0x00095103,
      19, QMetaType::UInt, 0x00095103,
      20, QMetaType::Bool, 0x00095103,
      21, QMetaType::QStringList, 0x00095001,

       0        // eod
};

void OrgBluezAdapter1Interface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OrgBluezAdapter1Interface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QDBusPendingReply<> _r = _t->RemoveDevice((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDBusPendingReply<> _r = _t->SetDiscoveryFilter((*reinterpret_cast< const QVariantMap(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<> _r = _t->StartDiscovery();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<> _r = _t->StopDiscovery();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<OrgBluezAdapter1Interface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->address(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->alias(); break;
        case 2: *reinterpret_cast< uint*>(_v) = _t->classProperty(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->discoverable(); break;
        case 4: *reinterpret_cast< uint*>(_v) = _t->discoverableTimeout(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->discovering(); break;
        case 6: *reinterpret_cast< QString*>(_v) = _t->modalias(); break;
        case 7: *reinterpret_cast< QString*>(_v) = _t->name(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->pairable(); break;
        case 9: *reinterpret_cast< uint*>(_v) = _t->pairableTimeout(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->powered(); break;
        case 11: *reinterpret_cast< QStringList*>(_v) = _t->uUIDs(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<OrgBluezAdapter1Interface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setAlias(*reinterpret_cast< QString*>(_v)); break;
        case 3: _t->setDiscoverable(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setDiscoverableTimeout(*reinterpret_cast< uint*>(_v)); break;
        case 8: _t->setPairable(*reinterpret_cast< bool*>(_v)); break;
        case 9: _t->setPairableTimeout(*reinterpret_cast< uint*>(_v)); break;
        case 10: _t->setPowered(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject OrgBluezAdapter1Interface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_meta_stringdata_OrgBluezAdapter1Interface.data,
    qt_meta_data_OrgBluezAdapter1Interface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OrgBluezAdapter1Interface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgBluezAdapter1Interface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OrgBluezAdapter1Interface.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgBluezAdapter1Interface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 12;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
