/****************************************************************************
** Meta object code from reading C++ file 'qnearfieldtarget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/nfc/qnearfieldtarget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qnearfieldtarget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QNearFieldTarget_t {
    QByteArrayData data[37];
    char stringdata0[508];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QNearFieldTarget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QNearFieldTarget_t qt_meta_stringdata_QNearFieldTarget = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QNearFieldTarget"
QT_MOC_LITERAL(1, 17, 12), // "disconnected"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 15), // "ndefMessageRead"
QT_MOC_LITERAL(4, 47, 12), // "QNdefMessage"
QT_MOC_LITERAL(5, 60, 7), // "message"
QT_MOC_LITERAL(6, 68, 19), // "ndefMessagesWritten"
QT_MOC_LITERAL(7, 88, 16), // "requestCompleted"
QT_MOC_LITERAL(8, 105, 27), // "QNearFieldTarget::RequestId"
QT_MOC_LITERAL(9, 133, 2), // "id"
QT_MOC_LITERAL(10, 136, 5), // "error"
QT_MOC_LITERAL(11, 142, 23), // "QNearFieldTarget::Error"
QT_MOC_LITERAL(12, 166, 14), // "handleResponse"
QT_MOC_LITERAL(13, 181, 8), // "response"
QT_MOC_LITERAL(14, 190, 4), // "Type"
QT_MOC_LITERAL(15, 195, 14), // "ProprietaryTag"
QT_MOC_LITERAL(16, 210, 11), // "NfcTagType1"
QT_MOC_LITERAL(17, 222, 11), // "NfcTagType2"
QT_MOC_LITERAL(18, 234, 11), // "NfcTagType3"
QT_MOC_LITERAL(19, 246, 11), // "NfcTagType4"
QT_MOC_LITERAL(20, 258, 9), // "MifareTag"
QT_MOC_LITERAL(21, 268, 12), // "AccessMethod"
QT_MOC_LITERAL(22, 281, 13), // "UnknownAccess"
QT_MOC_LITERAL(23, 295, 10), // "NdefAccess"
QT_MOC_LITERAL(24, 306, 21), // "TagTypeSpecificAccess"
QT_MOC_LITERAL(25, 328, 10), // "LlcpAccess"
QT_MOC_LITERAL(26, 339, 5), // "Error"
QT_MOC_LITERAL(27, 345, 7), // "NoError"
QT_MOC_LITERAL(28, 353, 12), // "UnknownError"
QT_MOC_LITERAL(29, 366, 16), // "UnsupportedError"
QT_MOC_LITERAL(30, 383, 21), // "TargetOutOfRangeError"
QT_MOC_LITERAL(31, 405, 15), // "NoResponseError"
QT_MOC_LITERAL(32, 421, 21), // "ChecksumMismatchError"
QT_MOC_LITERAL(33, 443, 22), // "InvalidParametersError"
QT_MOC_LITERAL(34, 466, 13), // "NdefReadError"
QT_MOC_LITERAL(35, 480, 14), // "NdefWriteError"
QT_MOC_LITERAL(36, 495, 12) // "CommandError"

    },
    "QNearFieldTarget\0disconnected\0\0"
    "ndefMessageRead\0QNdefMessage\0message\0"
    "ndefMessagesWritten\0requestCompleted\0"
    "QNearFieldTarget::RequestId\0id\0error\0"
    "QNearFieldTarget::Error\0handleResponse\0"
    "response\0Type\0ProprietaryTag\0NfcTagType1\0"
    "NfcTagType2\0NfcTagType3\0NfcTagType4\0"
    "MifareTag\0AccessMethod\0UnknownAccess\0"
    "NdefAccess\0TagTypeSpecificAccess\0"
    "LlcpAccess\0Error\0NoError\0UnknownError\0"
    "UnsupportedError\0TargetOutOfRangeError\0"
    "NoResponseError\0ChecksumMismatchError\0"
    "InvalidParametersError\0NdefReadError\0"
    "NdefWriteError\0CommandError"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QNearFieldTarget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       3,   62, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x06 /* Public */,
       3,    1,   45,    2, 0x06 /* Public */,
       6,    0,   48,    2, 0x06 /* Public */,
       7,    1,   49,    2, 0x06 /* Public */,
      10,    2,   52,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
      12,    2,   57,    2, 0x01 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, 0x80000000 | 11, 0x80000000 | 8,   10,    9,

 // methods: parameters
    QMetaType::Bool, 0x80000000 | 8, QMetaType::QByteArray,    9,   13,

 // enums: name, alias, flags, count, data
      14,   14, 0x0,    6,   77,
      21,   21, 0x0,    4,   89,
      26,   26, 0x0,   10,   97,

 // enum data: key, value
      15, uint(QNearFieldTarget::ProprietaryTag),
      16, uint(QNearFieldTarget::NfcTagType1),
      17, uint(QNearFieldTarget::NfcTagType2),
      18, uint(QNearFieldTarget::NfcTagType3),
      19, uint(QNearFieldTarget::NfcTagType4),
      20, uint(QNearFieldTarget::MifareTag),
      22, uint(QNearFieldTarget::UnknownAccess),
      23, uint(QNearFieldTarget::NdefAccess),
      24, uint(QNearFieldTarget::TagTypeSpecificAccess),
      25, uint(QNearFieldTarget::LlcpAccess),
      27, uint(QNearFieldTarget::NoError),
      28, uint(QNearFieldTarget::UnknownError),
      29, uint(QNearFieldTarget::UnsupportedError),
      30, uint(QNearFieldTarget::TargetOutOfRangeError),
      31, uint(QNearFieldTarget::NoResponseError),
      32, uint(QNearFieldTarget::ChecksumMismatchError),
      33, uint(QNearFieldTarget::InvalidParametersError),
      34, uint(QNearFieldTarget::NdefReadError),
      35, uint(QNearFieldTarget::NdefWriteError),
      36, uint(QNearFieldTarget::CommandError),

       0        // eod
};

void QNearFieldTarget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QNearFieldTarget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->disconnected(); break;
        case 1: _t->ndefMessageRead((*reinterpret_cast< const QNdefMessage(*)>(_a[1]))); break;
        case 2: _t->ndefMessagesWritten(); break;
        case 3: _t->requestCompleted((*reinterpret_cast< const QNearFieldTarget::RequestId(*)>(_a[1]))); break;
        case 4: _t->error((*reinterpret_cast< QNearFieldTarget::Error(*)>(_a[1])),(*reinterpret_cast< const QNearFieldTarget::RequestId(*)>(_a[2]))); break;
        case 5: { bool _r = _t->handleResponse((*reinterpret_cast< const QNearFieldTarget::RequestId(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QNearFieldTarget::RequestId >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QNearFieldTarget::RequestId >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QNearFieldTarget::RequestId >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QNearFieldTarget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QNearFieldTarget::disconnected)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QNearFieldTarget::*)(const QNdefMessage & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QNearFieldTarget::ndefMessageRead)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QNearFieldTarget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QNearFieldTarget::ndefMessagesWritten)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QNearFieldTarget::*)(const QNearFieldTarget::RequestId & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QNearFieldTarget::requestCompleted)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QNearFieldTarget::*)(QNearFieldTarget::Error , const QNearFieldTarget::RequestId & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QNearFieldTarget::error)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QNearFieldTarget::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QNearFieldTarget.data,
    qt_meta_data_QNearFieldTarget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QNearFieldTarget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QNearFieldTarget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QNearFieldTarget.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QNearFieldTarget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QNearFieldTarget::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QNearFieldTarget::ndefMessageRead(const QNdefMessage & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QNearFieldTarget::ndefMessagesWritten()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QNearFieldTarget::requestCompleted(const QNearFieldTarget::RequestId & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QNearFieldTarget::error(QNearFieldTarget::Error _t1, const QNearFieldTarget::RequestId & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
