/****************************************************************************
** Meta object code from reading C++ file 'qbluetoothlocaldevice.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/qbluetoothlocaldevice.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbluetoothlocaldevice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QBluetoothLocalDevice_t {
    QByteArrayData data[32];
    char stringdata0[488];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QBluetoothLocalDevice_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QBluetoothLocalDevice_t qt_meta_stringdata_QBluetoothLocalDevice = {
    {
QT_MOC_LITERAL(0, 0, 21), // "QBluetoothLocalDevice"
QT_MOC_LITERAL(1, 22, 20), // "hostModeStateChanged"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 31), // "QBluetoothLocalDevice::HostMode"
QT_MOC_LITERAL(4, 76, 5), // "state"
QT_MOC_LITERAL(5, 82, 15), // "deviceConnected"
QT_MOC_LITERAL(6, 98, 17), // "QBluetoothAddress"
QT_MOC_LITERAL(7, 116, 7), // "address"
QT_MOC_LITERAL(8, 124, 18), // "deviceDisconnected"
QT_MOC_LITERAL(9, 143, 15), // "pairingFinished"
QT_MOC_LITERAL(10, 159, 30), // "QBluetoothLocalDevice::Pairing"
QT_MOC_LITERAL(11, 190, 7), // "pairing"
QT_MOC_LITERAL(12, 198, 21), // "pairingDisplayPinCode"
QT_MOC_LITERAL(13, 220, 3), // "pin"
QT_MOC_LITERAL(14, 224, 26), // "pairingDisplayConfirmation"
QT_MOC_LITERAL(15, 251, 5), // "error"
QT_MOC_LITERAL(16, 257, 28), // "QBluetoothLocalDevice::Error"
QT_MOC_LITERAL(17, 286, 19), // "pairingConfirmation"
QT_MOC_LITERAL(18, 306, 12), // "confirmation"
QT_MOC_LITERAL(19, 319, 7), // "Pairing"
QT_MOC_LITERAL(20, 327, 8), // "Unpaired"
QT_MOC_LITERAL(21, 336, 6), // "Paired"
QT_MOC_LITERAL(22, 343, 16), // "AuthorizedPaired"
QT_MOC_LITERAL(23, 360, 8), // "HostMode"
QT_MOC_LITERAL(24, 369, 14), // "HostPoweredOff"
QT_MOC_LITERAL(25, 384, 15), // "HostConnectable"
QT_MOC_LITERAL(26, 400, 16), // "HostDiscoverable"
QT_MOC_LITERAL(27, 417, 30), // "HostDiscoverableLimitedInquiry"
QT_MOC_LITERAL(28, 448, 5), // "Error"
QT_MOC_LITERAL(29, 454, 7), // "NoError"
QT_MOC_LITERAL(30, 462, 12), // "PairingError"
QT_MOC_LITERAL(31, 475, 12) // "UnknownError"

    },
    "QBluetoothLocalDevice\0hostModeStateChanged\0"
    "\0QBluetoothLocalDevice::HostMode\0state\0"
    "deviceConnected\0QBluetoothAddress\0"
    "address\0deviceDisconnected\0pairingFinished\0"
    "QBluetoothLocalDevice::Pairing\0pairing\0"
    "pairingDisplayPinCode\0pin\0"
    "pairingDisplayConfirmation\0error\0"
    "QBluetoothLocalDevice::Error\0"
    "pairingConfirmation\0confirmation\0"
    "Pairing\0Unpaired\0Paired\0AuthorizedPaired\0"
    "HostMode\0HostPoweredOff\0HostConnectable\0"
    "HostDiscoverable\0HostDiscoverableLimitedInquiry\0"
    "Error\0NoError\0PairingError\0UnknownError"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QBluetoothLocalDevice[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       3,   84, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,
       5,    1,   57,    2, 0x06 /* Public */,
       8,    1,   60,    2, 0x06 /* Public */,
       9,    2,   63,    2, 0x06 /* Public */,
      12,    2,   68,    2, 0x06 /* Public */,
      14,    2,   73,    2, 0x06 /* Public */,
      15,    1,   78,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      17,    1,   81,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 10,    7,   11,
    QMetaType::Void, 0x80000000 | 6, QMetaType::QString,    7,   13,
    QMetaType::Void, 0x80000000 | 6, QMetaType::QString,    7,   13,
    QMetaType::Void, 0x80000000 | 16,   15,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,   18,

 // enums: name, alias, flags, count, data
      19,   19, 0x0,    3,   99,
      23,   23, 0x0,    4,  105,
      28,   28, 0x0,    3,  113,

 // enum data: key, value
      20, uint(QBluetoothLocalDevice::Unpaired),
      21, uint(QBluetoothLocalDevice::Paired),
      22, uint(QBluetoothLocalDevice::AuthorizedPaired),
      24, uint(QBluetoothLocalDevice::HostPoweredOff),
      25, uint(QBluetoothLocalDevice::HostConnectable),
      26, uint(QBluetoothLocalDevice::HostDiscoverable),
      27, uint(QBluetoothLocalDevice::HostDiscoverableLimitedInquiry),
      29, uint(QBluetoothLocalDevice::NoError),
      30, uint(QBluetoothLocalDevice::PairingError),
      31, uint(QBluetoothLocalDevice::UnknownError),

       0        // eod
};

void QBluetoothLocalDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QBluetoothLocalDevice *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->hostModeStateChanged((*reinterpret_cast< QBluetoothLocalDevice::HostMode(*)>(_a[1]))); break;
        case 1: _t->deviceConnected((*reinterpret_cast< const QBluetoothAddress(*)>(_a[1]))); break;
        case 2: _t->deviceDisconnected((*reinterpret_cast< const QBluetoothAddress(*)>(_a[1]))); break;
        case 3: _t->pairingFinished((*reinterpret_cast< const QBluetoothAddress(*)>(_a[1])),(*reinterpret_cast< QBluetoothLocalDevice::Pairing(*)>(_a[2]))); break;
        case 4: _t->pairingDisplayPinCode((*reinterpret_cast< const QBluetoothAddress(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 5: _t->pairingDisplayConfirmation((*reinterpret_cast< const QBluetoothAddress(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 6: _t->error((*reinterpret_cast< QBluetoothLocalDevice::Error(*)>(_a[1]))); break;
        case 7: _t->pairingConfirmation((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBluetoothLocalDevice::HostMode >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBluetoothAddress >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBluetoothAddress >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBluetoothAddress >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBluetoothLocalDevice::Pairing >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBluetoothAddress >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBluetoothAddress >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBluetoothLocalDevice::Error >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QBluetoothLocalDevice::*)(QBluetoothLocalDevice::HostMode );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBluetoothLocalDevice::hostModeStateChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QBluetoothLocalDevice::*)(const QBluetoothAddress & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBluetoothLocalDevice::deviceConnected)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QBluetoothLocalDevice::*)(const QBluetoothAddress & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBluetoothLocalDevice::deviceDisconnected)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QBluetoothLocalDevice::*)(const QBluetoothAddress & , QBluetoothLocalDevice::Pairing );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBluetoothLocalDevice::pairingFinished)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QBluetoothLocalDevice::*)(const QBluetoothAddress & , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBluetoothLocalDevice::pairingDisplayPinCode)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QBluetoothLocalDevice::*)(const QBluetoothAddress & , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBluetoothLocalDevice::pairingDisplayConfirmation)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QBluetoothLocalDevice::*)(QBluetoothLocalDevice::Error );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBluetoothLocalDevice::error)) {
                *result = 6;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QBluetoothLocalDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QBluetoothLocalDevice.data,
    qt_meta_data_QBluetoothLocalDevice,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QBluetoothLocalDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QBluetoothLocalDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QBluetoothLocalDevice.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QBluetoothLocalDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QBluetoothLocalDevice::hostModeStateChanged(QBluetoothLocalDevice::HostMode _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QBluetoothLocalDevice::deviceConnected(const QBluetoothAddress & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QBluetoothLocalDevice::deviceDisconnected(const QBluetoothAddress & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QBluetoothLocalDevice::pairingFinished(const QBluetoothAddress & _t1, QBluetoothLocalDevice::Pairing _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QBluetoothLocalDevice::pairingDisplayPinCode(const QBluetoothAddress & _t1, QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QBluetoothLocalDevice::pairingDisplayConfirmation(const QBluetoothAddress & _t1, QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QBluetoothLocalDevice::error(QBluetoothLocalDevice::Error _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
