/****************************************************************************
** Meta object code from reading C++ file 'hcimanager_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/bluez/hcimanager_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'hcimanager_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_HciManager_t {
    QByteArrayData data[21];
    char stringdata0[272];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_HciManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_HciManager_t qt_meta_stringdata_HciManager = {
    {
QT_MOC_LITERAL(0, 0, 10), // "HciManager"
QT_MOC_LITERAL(1, 11, 22), // "encryptionChangedEvent"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 17), // "QBluetoothAddress"
QT_MOC_LITERAL(4, 53, 7), // "address"
QT_MOC_LITERAL(5, 61, 10), // "wasSuccess"
QT_MOC_LITERAL(6, 72, 16), // "commandCompleted"
QT_MOC_LITERAL(7, 89, 6), // "opCode"
QT_MOC_LITERAL(8, 96, 6), // "status"
QT_MOC_LITERAL(9, 103, 4), // "data"
QT_MOC_LITERAL(10, 108, 18), // "connectionComplete"
QT_MOC_LITERAL(11, 127, 6), // "handle"
QT_MOC_LITERAL(12, 134, 16), // "connectionUpdate"
QT_MOC_LITERAL(13, 151, 30), // "QLowEnergyConnectionParameters"
QT_MOC_LITERAL(14, 182, 10), // "parameters"
QT_MOC_LITERAL(15, 193, 29), // "signatureResolvingKeyReceived"
QT_MOC_LITERAL(16, 223, 10), // "connHandle"
QT_MOC_LITERAL(17, 234, 9), // "remoteKey"
QT_MOC_LITERAL(18, 244, 8), // "quint128"
QT_MOC_LITERAL(19, 253, 4), // "csrk"
QT_MOC_LITERAL(20, 258, 13) // "_q_readNotify"

    },
    "HciManager\0encryptionChangedEvent\0\0"
    "QBluetoothAddress\0address\0wasSuccess\0"
    "commandCompleted\0opCode\0status\0data\0"
    "connectionComplete\0handle\0connectionUpdate\0"
    "QLowEnergyConnectionParameters\0"
    "parameters\0signatureResolvingKeyReceived\0"
    "connHandle\0remoteKey\0quint128\0csrk\0"
    "_q_readNotify"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_HciManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   44,    2, 0x06 /* Public */,
       6,    3,   49,    2, 0x06 /* Public */,
      10,    1,   56,    2, 0x06 /* Public */,
      12,    2,   59,    2, 0x06 /* Public */,
      15,    3,   64,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      20,    0,   71,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,    5,
    QMetaType::Void, QMetaType::UShort, QMetaType::UChar, QMetaType::QByteArray,    7,    8,    9,
    QMetaType::Void, QMetaType::UShort,   11,
    QMetaType::Void, QMetaType::UShort, 0x80000000 | 13,   11,   14,
    QMetaType::Void, QMetaType::UShort, QMetaType::Bool, 0x80000000 | 18,   16,   17,   19,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void HciManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HciManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->encryptionChangedEvent((*reinterpret_cast< const QBluetoothAddress(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->commandCompleted((*reinterpret_cast< quint16(*)>(_a[1])),(*reinterpret_cast< quint8(*)>(_a[2])),(*reinterpret_cast< const QByteArray(*)>(_a[3]))); break;
        case 2: _t->connectionComplete((*reinterpret_cast< quint16(*)>(_a[1]))); break;
        case 3: _t->connectionUpdate((*reinterpret_cast< quint16(*)>(_a[1])),(*reinterpret_cast< const QLowEnergyConnectionParameters(*)>(_a[2]))); break;
        case 4: _t->signatureResolvingKeyReceived((*reinterpret_cast< quint16(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< const quint128(*)>(_a[3]))); break;
        case 5: _t->_q_readNotify(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBluetoothAddress >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HciManager::*)(const QBluetoothAddress & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HciManager::encryptionChangedEvent)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (HciManager::*)(quint16 , quint8 , const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HciManager::commandCompleted)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (HciManager::*)(quint16 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HciManager::connectionComplete)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (HciManager::*)(quint16 , const QLowEnergyConnectionParameters & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HciManager::connectionUpdate)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (HciManager::*)(quint16 , bool , const quint128 & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HciManager::signatureResolvingKeyReceived)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject HciManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_HciManager.data,
    qt_meta_data_HciManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *HciManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HciManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_HciManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int HciManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void HciManager::encryptionChangedEvent(const QBluetoothAddress & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void HciManager::commandCompleted(quint16 _t1, quint8 _t2, const QByteArray & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void HciManager::connectionComplete(quint16 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void HciManager::connectionUpdate(quint16 _t1, const QLowEnergyConnectionParameters & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void HciManager::signatureResolvingKeyReceived(quint16 _t1, bool _t2, const quint128 & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
