/****************************************************************************
** Meta object code from reading C++ file 'obex_objectpush1_bluez5_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/bluez/obex_objectpush1_bluez5_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'obex_objectpush1_bluez5_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OrgBluezObexObjectPush1Interface_t {
    QByteArrayData data[12];
    char stringdata0[214];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OrgBluezObexObjectPush1Interface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OrgBluezObexObjectPush1Interface_t qt_meta_stringdata_OrgBluezObexObjectPush1Interface = {
    {
QT_MOC_LITERAL(0, 0, 32), // "OrgBluezObexObjectPush1Interface"
QT_MOC_LITERAL(1, 33, 21), // "ExchangeBusinessCards"
QT_MOC_LITERAL(2, 55, 46), // "QDBusPendingReply<QDBusObject..."
QT_MOC_LITERAL(3, 102, 0), // ""
QT_MOC_LITERAL(4, 103, 10), // "clientfile"
QT_MOC_LITERAL(5, 114, 10), // "targetfile"
QT_MOC_LITERAL(6, 125, 27), // "QDBusReply<QDBusObjectPath>"
QT_MOC_LITERAL(7, 153, 12), // "QVariantMap&"
QT_MOC_LITERAL(8, 166, 10), // "properties"
QT_MOC_LITERAL(9, 177, 16), // "PullBusinessCard"
QT_MOC_LITERAL(10, 194, 8), // "SendFile"
QT_MOC_LITERAL(11, 203, 10) // "sourcefile"

    },
    "OrgBluezObexObjectPush1Interface\0"
    "ExchangeBusinessCards\0"
    "QDBusPendingReply<QDBusObjectPath,QVariantMap>\0"
    "\0clientfile\0targetfile\0"
    "QDBusReply<QDBusObjectPath>\0QVariantMap&\0"
    "properties\0PullBusinessCard\0SendFile\0"
    "sourcefile"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OrgBluezObexObjectPush1Interface[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   44,    3, 0x0a /* Public */,
       1,    3,   49,    3, 0x0a /* Public */,
       9,    1,   56,    3, 0x0a /* Public */,
       9,    2,   59,    3, 0x0a /* Public */,
      10,    1,   64,    3, 0x0a /* Public */,
      10,    2,   67,    3, 0x0a /* Public */,

 // slots: parameters
    0x80000000 | 2, QMetaType::QString, QMetaType::QString,    4,    5,
    0x80000000 | 6, QMetaType::QString, QMetaType::QString, 0x80000000 | 7,    4,    5,    8,
    0x80000000 | 2, QMetaType::QString,    5,
    0x80000000 | 6, QMetaType::QString, 0x80000000 | 7,    5,    8,
    0x80000000 | 2, QMetaType::QString,   11,
    0x80000000 | 6, QMetaType::QString, 0x80000000 | 7,   11,    8,

       0        // eod
};

void OrgBluezObexObjectPush1Interface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OrgBluezObexObjectPush1Interface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QDBusPendingReply<QDBusObjectPath,QVariantMap> _r = _t->ExchangeBusinessCards((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QDBusObjectPath,QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDBusReply<QDBusObjectPath> _r = _t->ExchangeBusinessCards((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< QVariantMap(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QDBusReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<QDBusObjectPath,QVariantMap> _r = _t->PullBusinessCard((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QDBusObjectPath,QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusReply<QDBusObjectPath> _r = _t->PullBusinessCard((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QVariantMap(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<QDBusObjectPath,QVariantMap> _r = _t->SendFile((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QDBusObjectPath,QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusReply<QDBusObjectPath> _r = _t->SendFile((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QVariantMap(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject OrgBluezObexObjectPush1Interface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_meta_stringdata_OrgBluezObexObjectPush1Interface.data,
    qt_meta_data_OrgBluezObexObjectPush1Interface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OrgBluezObexObjectPush1Interface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgBluezObexObjectPush1Interface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OrgBluezObexObjectPush1Interface.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgBluezObexObjectPush1Interface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
