/****************************************************************************
** Meta object code from reading C++ file 'qbluetoothtransferreply_bluez_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/qbluetoothtransferreply_bluez_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbluetoothtransferreply_bluez_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QBluetoothTransferReplyBluez_t {
    QByteArrayData data[23];
    char stringdata0[271];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QBluetoothTransferReplyBluez_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QBluetoothTransferReplyBluez_t qt_meta_stringdata_QBluetoothTransferReplyBluez = {
    {
QT_MOC_LITERAL(0, 0, 28), // "QBluetoothTransferReplyBluez"
QT_MOC_LITERAL(1, 29, 5), // "start"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 8), // "copyDone"
QT_MOC_LITERAL(4, 45, 14), // "sessionCreated"
QT_MOC_LITERAL(5, 60, 24), // "QDBusPendingCallWatcher*"
QT_MOC_LITERAL(6, 85, 7), // "watcher"
QT_MOC_LITERAL(7, 93, 14), // "sessionStarted"
QT_MOC_LITERAL(8, 108, 14), // "sessionChanged"
QT_MOC_LITERAL(9, 123, 9), // "interface"
QT_MOC_LITERAL(10, 133, 18), // "changed_properties"
QT_MOC_LITERAL(11, 152, 22), // "invalidated_properties"
QT_MOC_LITERAL(12, 175, 12), // "QDBusMessage"
QT_MOC_LITERAL(13, 188, 5), // "abort"
QT_MOC_LITERAL(14, 194, 8), // "Complete"
QT_MOC_LITERAL(15, 203, 15), // "QDBusObjectPath"
QT_MOC_LITERAL(16, 219, 3), // "in0"
QT_MOC_LITERAL(17, 223, 5), // "Error"
QT_MOC_LITERAL(18, 229, 3), // "in1"
QT_MOC_LITERAL(19, 233, 8), // "Progress"
QT_MOC_LITERAL(20, 242, 7), // "Release"
QT_MOC_LITERAL(21, 250, 7), // "Request"
QT_MOC_LITERAL(22, 258, 12) // "sendReturned"

    },
    "QBluetoothTransferReplyBluez\0start\0\0"
    "copyDone\0sessionCreated\0"
    "QDBusPendingCallWatcher*\0watcher\0"
    "sessionStarted\0sessionChanged\0interface\0"
    "changed_properties\0invalidated_properties\0"
    "QDBusMessage\0abort\0Complete\0QDBusObjectPath\0"
    "in0\0Error\0in1\0Progress\0Release\0Request\0"
    "sendReturned"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QBluetoothTransferReplyBluez[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x08 /* Private */,
       3,    0,   75,    2, 0x08 /* Private */,
       4,    1,   76,    2, 0x08 /* Private */,
       7,    1,   79,    2, 0x08 /* Private */,
       8,    4,   82,    2, 0x08 /* Private */,
      13,    0,   91,    2, 0x0a /* Public */,
      14,    1,   92,    2, 0x0a /* Public */,
      17,    2,   95,    2, 0x0a /* Public */,
      19,    2,  100,    2, 0x0a /* Public */,
      20,    0,  105,    2, 0x0a /* Public */,
      21,    1,  106,    2, 0x0a /* Public */,
      22,    1,  109,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, QMetaType::QString, QMetaType::QVariantMap, QMetaType::QStringList, 0x80000000 | 12,    9,   10,   11,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void, 0x80000000 | 15, QMetaType::QString,   16,   18,
    QMetaType::Void, 0x80000000 | 15, QMetaType::ULongLong,   16,   18,
    QMetaType::Void,
    QMetaType::QString, 0x80000000 | 15,   16,
    QMetaType::Void, 0x80000000 | 5,    2,

       0        // eod
};

void QBluetoothTransferReplyBluez::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QBluetoothTransferReplyBluez *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { bool _r = _t->start();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->copyDone(); break;
        case 2: _t->sessionCreated((*reinterpret_cast< QDBusPendingCallWatcher*(*)>(_a[1]))); break;
        case 3: _t->sessionStarted((*reinterpret_cast< QDBusPendingCallWatcher*(*)>(_a[1]))); break;
        case 4: _t->sessionChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])),(*reinterpret_cast< const QDBusMessage(*)>(_a[4]))); break;
        case 5: _t->abort(); break;
        case 6: _t->Complete((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1]))); break;
        case 7: _t->Error((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 8: _t->Progress((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< qulonglong(*)>(_a[2]))); break;
        case 9: _t->Release(); break;
        case 10: { QString _r = _t->Request((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->sendReturned((*reinterpret_cast< QDBusPendingCallWatcher*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusPendingCallWatcher* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusPendingCallWatcher* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusMessage >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusPendingCallWatcher* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QBluetoothTransferReplyBluez::staticMetaObject = { {
    QMetaObject::SuperData::link<QBluetoothTransferReply::staticMetaObject>(),
    qt_meta_stringdata_QBluetoothTransferReplyBluez.data,
    qt_meta_data_QBluetoothTransferReplyBluez,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QBluetoothTransferReplyBluez::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QBluetoothTransferReplyBluez::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QBluetoothTransferReplyBluez.stringdata0))
        return static_cast<void*>(this);
    return QBluetoothTransferReply::qt_metacast(_clname);
}

int QBluetoothTransferReplyBluez::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QBluetoothTransferReply::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
