/****************************************************************************
** Meta object code from reading C++ file 'device_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/bluez/device_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'device_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OrgBluezDeviceInterface_t {
    QByteArrayData data[23];
    char stringdata0[383];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OrgBluezDeviceInterface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OrgBluezDeviceInterface_t qt_meta_stringdata_OrgBluezDeviceInterface = {
    {
QT_MOC_LITERAL(0, 0, 23), // "OrgBluezDeviceInterface"
QT_MOC_LITERAL(1, 24, 19), // "DisconnectRequested"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 11), // "NodeCreated"
QT_MOC_LITERAL(4, 57, 15), // "QDBusObjectPath"
QT_MOC_LITERAL(5, 73, 3), // "in0"
QT_MOC_LITERAL(6, 77, 11), // "NodeRemoved"
QT_MOC_LITERAL(7, 89, 15), // "PropertyChanged"
QT_MOC_LITERAL(8, 105, 12), // "QDBusVariant"
QT_MOC_LITERAL(9, 118, 3), // "in1"
QT_MOC_LITERAL(10, 122, 15), // "CancelDiscovery"
QT_MOC_LITERAL(11, 138, 19), // "QDBusPendingReply<>"
QT_MOC_LITERAL(12, 158, 10), // "CreateNode"
QT_MOC_LITERAL(13, 169, 34), // "QDBusPendingReply<QDBusObject..."
QT_MOC_LITERAL(14, 204, 10), // "Disconnect"
QT_MOC_LITERAL(15, 215, 16), // "DiscoverServices"
QT_MOC_LITERAL(16, 232, 29), // "QDBusPendingReply<ServiceMap>"
QT_MOC_LITERAL(17, 262, 13), // "GetProperties"
QT_MOC_LITERAL(18, 276, 30), // "QDBusPendingReply<QVariantMap>"
QT_MOC_LITERAL(19, 307, 9), // "ListNodes"
QT_MOC_LITERAL(20, 317, 42), // "QDBusPendingReply<QList<QDBus..."
QT_MOC_LITERAL(21, 360, 10), // "RemoveNode"
QT_MOC_LITERAL(22, 371, 11) // "SetProperty"

    },
    "OrgBluezDeviceInterface\0DisconnectRequested\0"
    "\0NodeCreated\0QDBusObjectPath\0in0\0"
    "NodeRemoved\0PropertyChanged\0QDBusVariant\0"
    "in1\0CancelDiscovery\0QDBusPendingReply<>\0"
    "CreateNode\0QDBusPendingReply<QDBusObjectPath>\0"
    "Disconnect\0DiscoverServices\0"
    "QDBusPendingReply<ServiceMap>\0"
    "GetProperties\0QDBusPendingReply<QVariantMap>\0"
    "ListNodes\0QDBusPendingReply<QList<QDBusObjectPath> >\0"
    "RemoveNode\0SetProperty"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OrgBluezDeviceInterface[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x06 /* Public */,
       3,    1,   75,    2, 0x06 /* Public */,
       6,    1,   78,    2, 0x06 /* Public */,
       7,    2,   81,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    0,   86,    2, 0x0a /* Public */,
      12,    1,   87,    2, 0x0a /* Public */,
      14,    0,   90,    2, 0x0a /* Public */,
      15,    1,   91,    2, 0x0a /* Public */,
      17,    0,   94,    2, 0x0a /* Public */,
      19,    0,   95,    2, 0x0a /* Public */,
      21,    1,   96,    2, 0x0a /* Public */,
      22,    2,   99,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 8,    5,    9,

 // slots: parameters
    0x80000000 | 11,
    0x80000000 | 13, QMetaType::QString,    5,
    0x80000000 | 11,
    0x80000000 | 16, QMetaType::QString,    5,
    0x80000000 | 18,
    0x80000000 | 20,
    0x80000000 | 11, 0x80000000 | 4,    5,
    0x80000000 | 11, QMetaType::QString, 0x80000000 | 8,    5,    9,

       0        // eod
};

void OrgBluezDeviceInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OrgBluezDeviceInterface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->DisconnectRequested(); break;
        case 1: _t->NodeCreated((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1]))); break;
        case 2: _t->NodeRemoved((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1]))); break;
        case 3: _t->PropertyChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QDBusVariant(*)>(_a[2]))); break;
        case 4: { QDBusPendingReply<> _r = _t->CancelDiscovery();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<QDBusObjectPath> _r = _t->CreateNode((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->Disconnect();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<ServiceMap> _r = _t->DiscoverServices((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<ServiceMap>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<QVariantMap> _r = _t->GetProperties();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<QList<QDBusObjectPath> > _r = _t->ListNodes();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QList<QDBusObjectPath> >*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<> _r = _t->RemoveNode((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<> _r = _t->SetProperty((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QDBusVariant(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusVariant >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusVariant >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (OrgBluezDeviceInterface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgBluezDeviceInterface::DisconnectRequested)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (OrgBluezDeviceInterface::*)(const QDBusObjectPath & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgBluezDeviceInterface::NodeCreated)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (OrgBluezDeviceInterface::*)(const QDBusObjectPath & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgBluezDeviceInterface::NodeRemoved)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (OrgBluezDeviceInterface::*)(const QString & , const QDBusVariant & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgBluezDeviceInterface::PropertyChanged)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject OrgBluezDeviceInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_meta_stringdata_OrgBluezDeviceInterface.data,
    qt_meta_data_OrgBluezDeviceInterface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OrgBluezDeviceInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgBluezDeviceInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OrgBluezDeviceInterface.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgBluezDeviceInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void OrgBluezDeviceInterface::DisconnectRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OrgBluezDeviceInterface::NodeCreated(const QDBusObjectPath & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void OrgBluezDeviceInterface::NodeRemoved(const QDBusObjectPath & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void OrgBluezDeviceInterface::PropertyChanged(const QString & _t1, const QDBusVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
