/****************************************************************************
** Meta object code from reading C++ file 'qlowenergycontroller_bluezdbus_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/qlowenergycontroller_bluezdbus_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qlowenergycontroller_bluezdbus_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QLowEnergyControllerPrivateBluezDBus_t {
    QByteArrayData data[19];
    char stringdata0[336];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QLowEnergyControllerPrivateBluezDBus_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QLowEnergyControllerPrivateBluezDBus_t qt_meta_stringdata_QLowEnergyControllerPrivateBluezDBus = {
    {
QT_MOC_LITERAL(0, 0, 36), // "QLowEnergyControllerPrivateBl..."
QT_MOC_LITERAL(1, 37, 23), // "devicePropertiesChanged"
QT_MOC_LITERAL(2, 61, 0), // ""
QT_MOC_LITERAL(3, 62, 9), // "interface"
QT_MOC_LITERAL(4, 72, 17), // "changedProperties"
QT_MOC_LITERAL(5, 90, 31), // "characteristicPropertiesChanged"
QT_MOC_LITERAL(6, 122, 16), // "QLowEnergyHandle"
QT_MOC_LITERAL(7, 139, 10), // "charHandle"
QT_MOC_LITERAL(8, 150, 21), // "invalidatedProperties"
QT_MOC_LITERAL(9, 172, 17), // "interfacesRemoved"
QT_MOC_LITERAL(10, 190, 15), // "QDBusObjectPath"
QT_MOC_LITERAL(11, 206, 10), // "objectPath"
QT_MOC_LITERAL(12, 217, 10), // "interfaces"
QT_MOC_LITERAL(13, 228, 18), // "onCharReadFinished"
QT_MOC_LITERAL(14, 247, 24), // "QDBusPendingCallWatcher*"
QT_MOC_LITERAL(15, 272, 4), // "call"
QT_MOC_LITERAL(16, 277, 18), // "onDescReadFinished"
QT_MOC_LITERAL(17, 296, 19), // "onCharWriteFinished"
QT_MOC_LITERAL(18, 316, 19) // "onDescWriteFinished"

    },
    "QLowEnergyControllerPrivateBluezDBus\0"
    "devicePropertiesChanged\0\0interface\0"
    "changedProperties\0characteristicPropertiesChanged\0"
    "QLowEnergyHandle\0charHandle\0"
    "invalidatedProperties\0interfacesRemoved\0"
    "QDBusObjectPath\0objectPath\0interfaces\0"
    "onCharReadFinished\0QDBusPendingCallWatcher*\0"
    "call\0onDescReadFinished\0onCharWriteFinished\0"
    "onDescWriteFinished"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QLowEnergyControllerPrivateBluezDBus[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   49,    2, 0x08 /* Private */,
       5,    4,   56,    2, 0x08 /* Private */,
       9,    2,   65,    2, 0x08 /* Private */,
      13,    1,   70,    2, 0x08 /* Private */,
      16,    1,   73,    2, 0x08 /* Private */,
      17,    1,   76,    2, 0x08 /* Private */,
      18,    1,   79,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QVariantMap, QMetaType::QStringList,    3,    4,    2,
    QMetaType::Void, 0x80000000 | 6, QMetaType::QString, QMetaType::QVariantMap, QMetaType::QStringList,    7,    3,    4,    8,
    QMetaType::Void, 0x80000000 | 10, QMetaType::QStringList,   11,   12,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void, 0x80000000 | 14,   15,

       0        // eod
};

void QLowEnergyControllerPrivateBluezDBus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QLowEnergyControllerPrivateBluezDBus *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->devicePropertiesChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3]))); break;
        case 1: _t->characteristicPropertiesChanged((*reinterpret_cast< QLowEnergyHandle(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QVariantMap(*)>(_a[3])),(*reinterpret_cast< const QStringList(*)>(_a[4]))); break;
        case 2: _t->interfacesRemoved((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2]))); break;
        case 3: _t->onCharReadFinished((*reinterpret_cast< QDBusPendingCallWatcher*(*)>(_a[1]))); break;
        case 4: _t->onDescReadFinished((*reinterpret_cast< QDBusPendingCallWatcher*(*)>(_a[1]))); break;
        case 5: _t->onCharWriteFinished((*reinterpret_cast< QDBusPendingCallWatcher*(*)>(_a[1]))); break;
        case 6: _t->onDescWriteFinished((*reinterpret_cast< QDBusPendingCallWatcher*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QLowEnergyControllerPrivateBluezDBus::staticMetaObject = { {
    QMetaObject::SuperData::link<QLowEnergyControllerPrivate::staticMetaObject>(),
    qt_meta_stringdata_QLowEnergyControllerPrivateBluezDBus.data,
    qt_meta_data_QLowEnergyControllerPrivateBluezDBus,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QLowEnergyControllerPrivateBluezDBus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QLowEnergyControllerPrivateBluezDBus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QLowEnergyControllerPrivateBluezDBus.stringdata0))
        return static_cast<void*>(this);
    return QLowEnergyControllerPrivate::qt_metacast(_clname);
}

int QLowEnergyControllerPrivateBluezDBus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLowEnergyControllerPrivate::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
