/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativebluetoothdiscoverymodel_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../git/src/imports/bluetooth/qdeclarativebluetoothdiscoverymodel_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativebluetoothdiscoverymodel_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QDeclarativeBluetoothDiscoveryModel_t {
    QByteArrayData data[34];
    char stringdata0[596];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QDeclarativeBluetoothDiscoveryModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QDeclarativeBluetoothDiscoveryModel_t qt_meta_stringdata_QDeclarativeBluetoothDiscoveryModel = {
    {
QT_MOC_LITERAL(0, 0, 35), // "QDeclarativeBluetoothDiscover..."
QT_MOC_LITERAL(1, 36, 12), // "errorChanged"
QT_MOC_LITERAL(2, 49, 0), // ""
QT_MOC_LITERAL(3, 50, 20), // "discoveryModeChanged"
QT_MOC_LITERAL(4, 71, 17), // "serviceDiscovered"
QT_MOC_LITERAL(5, 89, 29), // "QDeclarativeBluetoothService*"
QT_MOC_LITERAL(6, 119, 7), // "service"
QT_MOC_LITERAL(7, 127, 16), // "deviceDiscovered"
QT_MOC_LITERAL(8, 144, 6), // "device"
QT_MOC_LITERAL(9, 151, 14), // "runningChanged"
QT_MOC_LITERAL(10, 166, 17), // "uuidFilterChanged"
QT_MOC_LITERAL(11, 184, 20), // "remoteAddressChanged"
QT_MOC_LITERAL(12, 205, 21), // "QBluetoothServiceInfo"
QT_MOC_LITERAL(13, 227, 20), // "QBluetoothDeviceInfo"
QT_MOC_LITERAL(14, 248, 17), // "finishedDiscovery"
QT_MOC_LITERAL(15, 266, 14), // "errorDiscovery"
QT_MOC_LITERAL(16, 281, 38), // "QBluetoothServiceDiscoveryAge..."
QT_MOC_LITERAL(17, 320, 5), // "error"
QT_MOC_LITERAL(18, 326, 20), // "errorDeviceDiscovery"
QT_MOC_LITERAL(19, 347, 37), // "QBluetoothDeviceDiscoveryAgen..."
QT_MOC_LITERAL(20, 385, 5), // "Error"
QT_MOC_LITERAL(21, 391, 13), // "discoveryMode"
QT_MOC_LITERAL(22, 405, 13), // "DiscoveryMode"
QT_MOC_LITERAL(23, 419, 7), // "running"
QT_MOC_LITERAL(24, 427, 10), // "uuidFilter"
QT_MOC_LITERAL(25, 438, 13), // "remoteAddress"
QT_MOC_LITERAL(26, 452, 23), // "MinimalServiceDiscovery"
QT_MOC_LITERAL(27, 476, 20), // "FullServiceDiscovery"
QT_MOC_LITERAL(28, 497, 15), // "DeviceDiscovery"
QT_MOC_LITERAL(29, 513, 7), // "NoError"
QT_MOC_LITERAL(30, 521, 16), // "InputOutputError"
QT_MOC_LITERAL(31, 538, 15), // "PoweredOffError"
QT_MOC_LITERAL(32, 554, 12), // "UnknownError"
QT_MOC_LITERAL(33, 567, 28) // "InvalidBluetoothAdapterError"

    },
    "QDeclarativeBluetoothDiscoveryModel\0"
    "errorChanged\0\0discoveryModeChanged\0"
    "serviceDiscovered\0QDeclarativeBluetoothService*\0"
    "service\0deviceDiscovered\0device\0"
    "runningChanged\0uuidFilterChanged\0"
    "remoteAddressChanged\0QBluetoothServiceInfo\0"
    "QBluetoothDeviceInfo\0finishedDiscovery\0"
    "errorDiscovery\0QBluetoothServiceDiscoveryAgent::Error\0"
    "error\0errorDeviceDiscovery\0"
    "QBluetoothDeviceDiscoveryAgent::Error\0"
    "Error\0discoveryMode\0DiscoveryMode\0"
    "running\0uuidFilter\0remoteAddress\0"
    "MinimalServiceDiscovery\0FullServiceDiscovery\0"
    "DeviceDiscovery\0NoError\0InputOutputError\0"
    "PoweredOffError\0UnknownError\0"
    "InvalidBluetoothAdapterError"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QDeclarativeBluetoothDiscoveryModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       5,   98, // properties
       2,  118, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x06 /* Public */,
       3,    0,   75,    2, 0x06 /* Public */,
       4,    1,   76,    2, 0x06 /* Public */,
       7,    1,   79,    2, 0x06 /* Public */,
       9,    0,   82,    2, 0x06 /* Public */,
      10,    0,   83,    2, 0x06 /* Public */,
      11,    0,   84,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   85,    2, 0x08 /* Private */,
       7,    1,   88,    2, 0x08 /* Private */,
      14,    0,   91,    2, 0x08 /* Private */,
      15,    1,   92,    2, 0x08 /* Private */,
      18,    1,   95,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 12,    6,
    QMetaType::Void, 0x80000000 | 13,    8,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, 0x80000000 | 19,    2,

 // properties: name, type, flags
      17, 0x80000000 | 20, 0x00495009,
      21, 0x80000000 | 22, 0x0049510b,
      23, QMetaType::Bool, 0x00495103,
      24, QMetaType::QString, 0x00495103,
      25, QMetaType::QString, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       4,
       5,
       6,

 // enums: name, alias, flags, count, data
      22,   22, 0x0,    3,  128,
      20,   20, 0x0,    5,  134,

 // enum data: key, value
      26, uint(QDeclarativeBluetoothDiscoveryModel::MinimalServiceDiscovery),
      27, uint(QDeclarativeBluetoothDiscoveryModel::FullServiceDiscovery),
      28, uint(QDeclarativeBluetoothDiscoveryModel::DeviceDiscovery),
      29, uint(QDeclarativeBluetoothDiscoveryModel::NoError),
      30, uint(QDeclarativeBluetoothDiscoveryModel::InputOutputError),
      31, uint(QDeclarativeBluetoothDiscoveryModel::PoweredOffError),
      32, uint(QDeclarativeBluetoothDiscoveryModel::UnknownError),
      33, uint(QDeclarativeBluetoothDiscoveryModel::InvalidBluetoothAdapterError),

       0        // eod
};

void QDeclarativeBluetoothDiscoveryModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QDeclarativeBluetoothDiscoveryModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->errorChanged(); break;
        case 1: _t->discoveryModeChanged(); break;
        case 2: _t->serviceDiscovered((*reinterpret_cast< QDeclarativeBluetoothService*(*)>(_a[1]))); break;
        case 3: _t->deviceDiscovered((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->runningChanged(); break;
        case 5: _t->uuidFilterChanged(); break;
        case 6: _t->remoteAddressChanged(); break;
        case 7: _t->serviceDiscovered((*reinterpret_cast< const QBluetoothServiceInfo(*)>(_a[1]))); break;
        case 8: _t->deviceDiscovered((*reinterpret_cast< const QBluetoothDeviceInfo(*)>(_a[1]))); break;
        case 9: _t->finishedDiscovery(); break;
        case 10: _t->errorDiscovery((*reinterpret_cast< QBluetoothServiceDiscoveryAgent::Error(*)>(_a[1]))); break;
        case 11: _t->errorDeviceDiscovery((*reinterpret_cast< QBluetoothDeviceDiscoveryAgent::Error(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeBluetoothService* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBluetoothServiceInfo >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBluetoothDeviceInfo >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QDeclarativeBluetoothDiscoveryModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeBluetoothDiscoveryModel::errorChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QDeclarativeBluetoothDiscoveryModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeBluetoothDiscoveryModel::discoveryModeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QDeclarativeBluetoothDiscoveryModel::*)(QDeclarativeBluetoothService * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeBluetoothDiscoveryModel::serviceDiscovered)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QDeclarativeBluetoothDiscoveryModel::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeBluetoothDiscoveryModel::deviceDiscovered)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QDeclarativeBluetoothDiscoveryModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeBluetoothDiscoveryModel::runningChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QDeclarativeBluetoothDiscoveryModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeBluetoothDiscoveryModel::uuidFilterChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QDeclarativeBluetoothDiscoveryModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeBluetoothDiscoveryModel::remoteAddressChanged)) {
                *result = 6;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QDeclarativeBluetoothDiscoveryModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< Error*>(_v) = _t->error(); break;
        case 1: *reinterpret_cast< DiscoveryMode*>(_v) = _t->discoveryMode(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->running(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->uuidFilter(); break;
        case 4: *reinterpret_cast< QString*>(_v) = _t->remoteAddress(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QDeclarativeBluetoothDiscoveryModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setDiscoveryMode(*reinterpret_cast< DiscoveryMode*>(_v)); break;
        case 2: _t->setRunning(*reinterpret_cast< bool*>(_v)); break;
        case 3: _t->setUuidFilter(*reinterpret_cast< QString*>(_v)); break;
        case 4: _t->setRemoteAddress(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QDeclarativeBluetoothDiscoveryModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_meta_stringdata_QDeclarativeBluetoothDiscoveryModel.data,
    qt_meta_data_QDeclarativeBluetoothDiscoveryModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QDeclarativeBluetoothDiscoveryModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeBluetoothDiscoveryModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QDeclarativeBluetoothDiscoveryModel.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int QDeclarativeBluetoothDiscoveryModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QDeclarativeBluetoothDiscoveryModel::errorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeBluetoothDiscoveryModel::discoveryModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeBluetoothDiscoveryModel::serviceDiscovered(QDeclarativeBluetoothService * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QDeclarativeBluetoothDiscoveryModel::deviceDiscovered(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QDeclarativeBluetoothDiscoveryModel::runningChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDeclarativeBluetoothDiscoveryModel::uuidFilterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QDeclarativeBluetoothDiscoveryModel::remoteAddressChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
