/****************************************************************************
** Meta object code from reading C++ file 'obex_agent_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/bluez/obex_agent_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'obex_agent_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_AgentAdaptor_t {
    QByteArrayData data[14];
    char stringdata0[704];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_AgentAdaptor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_AgentAdaptor_t qt_meta_stringdata_AgentAdaptor = {
    {
QT_MOC_LITERAL(0, 0, 12), // "AgentAdaptor"
QT_MOC_LITERAL(1, 13, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 29, 18), // "org.openobex.Agent"
QT_MOC_LITERAL(3, 48, 19), // "D-Bus Introspection"
QT_MOC_LITERAL(4, 68, 570), // "  <interface name=\"org.openo..."
QT_MOC_LITERAL(5, 581, 8), // "Complete"
QT_MOC_LITERAL(6, 590, 0), // ""
QT_MOC_LITERAL(7, 591, 15), // "QDBusObjectPath"
QT_MOC_LITERAL(8, 607, 3), // "in0"
QT_MOC_LITERAL(9, 611, 5), // "Error"
QT_MOC_LITERAL(10, 617, 3), // "in1"
QT_MOC_LITERAL(11, 621, 8), // "Progress"
QT_MOC_LITERAL(12, 630, 7), // "Release"
QT_MOC_LITERAL(13, 638, 7) // "Request"

    },
    "AgentAdaptor\0D-Bus Interface\0"
    "org.openobex.Agent\0D-Bus Introspection\0"
    "  <interface name=\"org.openobex.Agent\">\n    <method name=\"Release\""
    "/>\n    <method name=\"Request\">\n      <arg direction=\"in\" type=\""
    "o\"/>\n      <arg direction=\"out\" type=\"s\"/>\n    </method>\n    <"
    "method name=\"Progress\">\n      <arg direction=\"in\" type=\"o\"/>\n "
    "     <arg direction=\"in\" type=\"t\"/>\n    </method>\n    <method na"
    "me=\"Complete\">\n      <arg direction=\"in\" type=\"o\"/>\n    </meth"
    "od>\n    <method name=\"Error\">\n      <arg direction=\"in\" type=\"o"
    "\"/>\n      <arg direction=\"in\" type=\"s\"/>\n    </method>\n  </int"
    "erface>\n\0"
    "Complete\0\0QDBusObjectPath\0in0\0Error\0"
    "in1\0Progress\0Release\0Request"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_AgentAdaptor[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       5,   18, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // slots: name, argc, parameters, tag, flags
       5,    1,   43,    6, 0x0a /* Public */,
       9,    2,   46,    6, 0x0a /* Public */,
      11,    2,   51,    6, 0x0a /* Public */,
      12,    0,   56,    6, 0x0a /* Public */,
      13,    1,   57,    6, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 7, QMetaType::QString,    8,   10,
    QMetaType::Void, 0x80000000 | 7, QMetaType::ULongLong,    8,   10,
    QMetaType::Void,
    QMetaType::QString, 0x80000000 | 7,    8,

       0        // eod
};

void AgentAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AgentAdaptor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->Complete((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1]))); break;
        case 1: _t->Error((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 2: _t->Progress((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< qulonglong(*)>(_a[2]))); break;
        case 3: _t->Release(); break;
        case 4: { QString _r = _t->Request((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject AgentAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_meta_stringdata_AgentAdaptor.data,
    qt_meta_data_AgentAdaptor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *AgentAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AgentAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_AgentAdaptor.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int AgentAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
